;*****************************************************
;                                                   ;
;      BIOS60  MOPPEL-60-K-BIOS  V 12.69             ;
;                                                   ;
;*****************************************************
;
; HMS-Quelle Stand:   Mo,02.03.87;19:43:10h
;
;{
; Ueberarbeitet und Anpassung auf 8085Simulator-ID
;				https://www.oshonsoft.com/
;				notepad++ Tab = 6
;				(werner.roemer@t-online.de)
;
; 16.08.2023	IDE Modul OK
; 07.08.2023	Einbau IDE-Laufwerk
;			Tracknummer auf 16bit erweitert
;			(iotrk u. akttrk)
;
; 03.08.2023 Eingehende Analyse vom Disketten-Management
;		 *
;		 * BIOS kann ein und zweiseitige Disketten
;		 * verarbeiten
;		 * bis zu 2x80Tr, DD, 16Sek a256byte
;		 *
;		 * die 128Byte (Records) vom BDOS werden im
;		 * 1k Buffer abgebildet = 4 phys. Sektoren
;		 *
;		 * die 2. Seite wird ab Track 50h absteigend
;		 * auf Seite 1 abgebildet
;		 *
;		 * das Format wird ueber die Drive Parameter
;		 * bestimmt
;		 *
;		 * Byte 0 => 00=40Trk/1kBlock
;		 *		 01=80Trk/2kBlock
;		 *		 02=2x80Trk/2kBlock
;		 *		 03=  dito /4kBlock
;		 * Byte 1 => 00=6ms Steprate
;		 *		 08=12ms
;		 *		 10=20ms
;		 *		 18=30ms
;		 * Byte 2 => 00=einseitig
;		 *		 01=zweiseitig
;		 * Byte 3 => Tracks pro Spur
;		 *		 max 28h bzw 50h
;		 *		 ! als max +1 angeben
;		 *
;		 * mit dem Bootloader in den Parameterbereich
;		 * geladen + den zugehoerigen DPB
;		 *
;		 *
;
; 02.08.2023	Gliederung fuer Notepad eingebaut
;
; 01.07.2023	Quellen von HMS aufbereitet
;
;}
;;*****************************************************
;
; Zuweisungen
;
;;*****************************************************
;{
;
buffer 	equ	0f800h	;2 KB bis MEM Top
verbuf 	equ	0f700h
variab 	equ	0f500h
syspar 	equ	0f400h	;vom Urloader
;
bios   	equ	0ea00h
bdos   	equ	0dc06h
ccp    	equ	0d400h
;
stacku 	equ	verbuf
stackc 	equ	 100h
stackw 	equ	  80h
buff   	equ	  80h
cdisk  	equ	   4h		;Kaltstart-Drive/User
iobyte 	equ	   3h
;
;
rambnk 	equ	  24h		;Banking-Latch
;      	equ
cimo   	equ	  43h		;Monitor-Unterprogramme
como   	equ	  49h
cstsm  	equ	  52h
cstsmo 	equ	100ch
lomo   	equ	  4fh
v24imo 	equ	1024h
v24omo 	equ	101eh
vinimo 	equ	  64h		;Adr.!
pinimo 	equ	  58h
;
stack0 	equ	2950h		;Bank-0-Stack
ioflag 	equ	2fc8h		;LO Select
;
;
begin  	equ	0101h		;Systemspuren ab Trk.1, Sec.1
nkalt  	equ	  1fh		;Anzahl Sektoren CCP/BDOS/BIOS
nwarm  	equ	  16h		;Anzahl Sektoren CCP/BDOS
offset 	equ	  04h		;Start bei Track 4
nmax   	equ	  04h		;LW-Anzahl
rwdhg  	equ	   5h		;Wiederholungen bei R/W-Fehler
timout 	equ	   6h		;max.Positionierzeit (x0,5 s)
;
;}
;
;*****************************************************
;
; BIOS Sprungtabelle
;
;*****************************************************
;{
		org   bios		;BIOS Start

		jmp  cboot 		;Kaltstart
		jmp  wboot 		;Warmstart
		jmp  const 		;Console Status
		jmp  conin 		;Console In
		jmp  conout		;Console Out
		jmp  list  		;Lineprinter
		jmp  punch 		;Puncher Out
		jmp  reader		;Reader In
		jmp  home  		;Track 0 einstellen
		jmp  seldsk		;Select Disk
		jmp  settrk		;Set Track No.
		jmp  setsec		;Set Sector No.
		jmp  setdma		;Set Disk Buffer
		jmp  read  		;Read Logical Sector
		jmp  write 		;Write Logical Sector
		jmp  lsts  		;Lineprinter Status
		jmp  sectra		;Get Translate Table
		jmp  prtini		;Schnittstellen initialisieren
		jmp  v24ini
;
;}
;
;*****************************************************
;
;     Disk Parameter Header
;
;*****************************************************
;{
dpbase:	ds   0h    		;Variablen und Tabellen am Ende
;
dpe0:  	dw   0h    		;Translate Table (XLT)
		dw   0h    		;Highest Dir.Entry
		dw   0h    		;Current Logical Track
		dw   0h    		;First Sector of Current Track
		dw   dirbuf		;Directory Buffer
		dw   dpb0  		;Disk Parameter Block
		dw   csv0  		;Checksum Vector
		dw   alv0  		;Allocation Vector
;
dpe1:  	dw   0h
		dw   0h
		dw   0h
		dw   0h
		dw   dirbuf
		dw   dpb1
		dw   csv1
		dw   alv1
;
dpe2:  	dw   0h
		dw   0h
		dw   0h
		dw   0h
		dw   dirbuf
		dw   dpb2
		dw   csv2
		dw   alv2
;
dpe3:  	dw   0h
		dw   0h
		dw   0h
		dw   0h
		dw   dirbuf
		dw   dpb3
		dw   csv3
		dw   alv3
;
;}
;
;*****************************************************
;
;      Peripherie-Ansprache
;
;*****************************************************
;{
conout:	lxi  h,cotab
		jmp  consol
conin: 	lxi  h,citab
consol:	lda  iobyte    ;CONSOLE (I/O-Bits 0&1)
		ani  3h
		jmp  decode
cotab: 	dw   co        ;Video Out
		dw   crtout
		dw   list
		dw   uscn3o
citab: 	dw   ci        ;ASCII In
		dw   crtin
		dw   reader
		dw   uscn3i
;
const: 	lda  iobyte    ;Console Status
		ani  3h
		lxi  h,cistab
		jmp  decode
cistab:	dw   csts      ;ASCII Status
		dw   crtst
		dw   ristat
		dw   uscn3s
;
;***
;
reader:	lda  iobyte    ;READER IN (I/O-Bits 2&3)
		rrc
		rrc
		ani  3h
		lxi  h,ritab
		jmp  decode
ritab: 	dw   ci        ;ASCII Input
		dw   v24in
		dw   usri2
		dw   usri3
;
ristat:	lda  iobyte    ;Reader Status
		rrc
		rrc
		ani  3h
		lxi  h,ristab
		jmp  decode
ristab:	dw   csts      ;ASCII Status
		dw   v24st
		dw   usri2s
		dw   usri3s
;
;***
;
punch: 	lda  iobyte    ;PUNCHER OUT (I/O-Bits 4&5)
		rrc
		rrc
		rrc
		rrc
		ani  3h
		lxi  h,potab
		jmp  decode
potab: 	dw   co        ;Video Out
		dw   v24out
		dw   uspo2
		dw   uspo3
;
pustat:	lda  iobyte    ;Puncher Status
		rrc
		rrc
		rrc
		rrc
		ani  3h
		lxi  h,pustab
		jmp  decode
pustab:	dw   rdysta    ;ready
		dw   v24st
		dw   uspo2s
		dw   uspo3s
;
;***
;
list:  	lda  iobyte    ;LISTER OUT (I/O-Bits 6&7)
		rlc
		rlc
		ani  3h
		lxi  h,lotab
		jmp  decode
lotab: 	dw   co        ;Video Out
		dw   crtout
		dw   prt
		dw   centr
;
lsts:  	lda  iobyte    ;Lister Status
		rlc
		rlc
		ani  3h
		lxi  h,listab
		jmp  decode
listab:	dw   rdysta    ;ready
		dw   crtst
		dw   prtst
		dw   censta
;
;
decode:	jz   found     ;Verzweigung nach Tabelle
		inx  h
		inx  h
		dcr  a
		jmp  decode
found: 	mov  e,m
		inx  h
		mov  d,m
		xchg           ;H&L: Zieladresse
		pchl
;}
;
;*****************************************************
;
;      Monitor-Unterprogramme (incl.User)
;
;*****************************************************
;{
monio: 	shld monadr    ;universelles Ein/Ausgabe-UP
		lxi  h,0h
		dad  sp        ;H&L: Bank 1 Stack
		lxi  d,stack0  ;D&E: Bank 0 Stack
		xchg
		sphl           ;Stack Pointer auf Bank 0 Stack
		mvi  a,81h
		out  rambnk    ;umschalten auf Bank 0
		push d         ;Bank 1 Stack retten
;
		db   0cdh      ;CALL
monadr:	ds   2h        ;UP-Startadresse
;
		mov  c,a
		pop  h         ;Bank 1 Stack zurueck
		mvi  a,1h
		out  rambnk    ;Umschalten auf Bank 1
		sphl           ;alter Bank 1 Stack
		pop  h         ;Restore Regs
		pop  d
		mov  a,c
		ret
;
;
ci:    	push d         ;ASCII Input
		push h
		lxi  h,cisub
		jmp  monio
cisub: 	call cimo
		push psw
cilop: 	call cstsmo    ;auf Loslassen warten
		jnz  cilop
		pop  psw
		ret
;
;
co:    	push d         ;Video Output
		push h
		lxi  h,como
		jmp  monio
;
;
csts:  	push d         ;ASCII Status
		push h
		lxi  h,cstsm
		jmp  monio
;
;
prt:   	push d         ;Drucker (seriell)
		push h
		lxi  h,prtsub
		jmp  monio
prtsub:	mvi  a,80h     ;d.h. LO7
prtret:	sta  ioflag
		call lomo
		xra  a
		sta  ioflag
		ret
;
;
centr: 	push d         ;Drucker (parallel)
		push h
		lxi  h,censub
		jmp  monio
censub:	mvi  a,40h     ;d.h. LO6
		jmp  prtret
;
;
v24in: 	push d         ;V.24 Input
		push h
		lxi  h,v24imo
		jmp  monio
;
;
v24out:	push d         ;V.24 Output
		push h
		lxi  h,v24omo
		jmp  monio
;
;
v24ini:	push d         ;V.24 initialisieren
		push h
		lhld vinimo    ;Adr!
		jmp  monio
;
;
prtini:	push d         ;SER initialisieren
		push h
		lxi  h,pinimo
		jmp  monio
;
;
crtin: 	push d         ;Terminal Input
		push h
		lhld usadr0
		jmp  monio
;
;
crtout:	push d         ;Terminal Output
		push h
		lhld usadr1
		jmp  monio
;
;
uscn3i:	push d         ;User Console Input
		push h
		lhld usadr2
		jmp  monio
;
;
uscn3o:	push d         ;User Console Output
		push h
		lhld usadr3
		jmp  monio
;
;
usri2: 	push d         ;User Reader Input (1)
		push h
		lhld usadr4
		jmp  monio
;
;
usri3: 	push d         ;User Reader Input (2)
		push h
		lhld usadr5
		jmp  monio
;
;
uspo2: 	push d         ;User Puncher Out (1)
		push h
		lhld usadr6
		jmp  monio
;
;
uspo3: 	push d         ;User Puncher Out (2)
		push h
		lhld usadr7
		jmp  monio
;
;
prtst: 	ds 0h
censta:	ds 0h
v24st: 	ds 0h
crtst: 	ds 0h
uscn3s:	ds 0h
usri2s:	ds 0h
usri3s:	ds 0h
uspo2s:	ds 0h
uspo3s:	ds 0h
;
rdysta:	xra  a         ;Dummy Status: Device Ready
		dcr  a         ;A: FFh
		ret            ;ZERO off
;}
;
;*****************************************************
;
;	Disketten-Management
;	Uebernahme der BDOS Parameter
;
;*****************************************************
;{
;
;*** Home/Set Track	{
;
; I:
; siehe Set Track
;
; BIOS-Funktion Set Track
; 16Bit
;
; I:	(BC)	=	Track
; O:
;
home:  	lxi  b,0h      	;Track 0 einstellen
settrk:	push b         	;Trackbuffer laden
		pop  h
		shld iotrk
		ret
;		}
;
;*** Set Sector	{
;
; log.Sektor 1-32 wg. Block/deblock
;
; I:	(C)	=	Sektor
;
;
setsec:	push b         	; Sectorbuffer laden
		pop  h
		shld iosec
		ret
;		}
;
;*** Set DMA	{
;
; I:	(BC)	=	DMA Adresse
;
setdma:	push b         	; DMA-Adresse laden
		pop  h
		shld iodma
		ret
;		}
;
;*** Selekt Drive	{
;
; I:	(C)	=	Laufwerk
; O:	(HL)	= 	DPH	(HL)=0 Laufwerk nicht vorhanden
;
seldsk:	lxi  h,0h      	; fuer Fehlerbehandlung
		mov  a,c
		cpi  nmax      	; LW im Bereich?
		jc   selcte    	; ja:JMP
		lda  sysusr
		sta  cdisk
		ret            	; H&L=0 --> Select Fehler
;
selcte:	sta  iodrv     	; LW-Nr.
		call parcop		; LW Parameter kopieren
		add  a
		add  a
		add  a
		add  a         	; DPB-Laenge: 16d
		mov  l,a
		lxi  d,dpbase
		dad  d         	; H&L: DPEx-Anfangsadresse
		ret
;		}
;
;*** Test ob Translate erforderlich
; im Moppel nicht benutzt
;{
;
sectra:	push b        	; in H&L Sektor-Nr.Uebergeben
		pop  h
		mov  a,d
		ora  e
		rz             	; No X-late Table
		dad  d
		mov  e,m       	; Wert aus Translate Table
		mvi  d,0h
		xchg
		ret
;		}
;
;*** Read log. Sector	{
;
; I:	vorher alle SET-Befehle Drive, Track, Sektor, DMA
; O:	log.Sektor von Buffer zum BDOS DMA
;	erst wenn Buffer ungueltig ist erfolgt der
;	physikalische Laufwerkszugriff
;
read:  	call calcmp    	; Block lesen
		cnz  rload
		call bufcal
		call copy
		xra  a         	; o.k.:ACC=00h
		ret	            ; ==> zurueck zum BDOS
;
rload: 	lda  wrtflg
		ora  a         	; Buffer leer?
		jz   rlod      	; ja: JMP
		call puttrk    	; sonst abspeichern
		lda  iodrv
		call parcop
rlod:  	call gettrk
		ret			; ==> zurueck zum BDOS
;		}
;
;*** Write log.Sector	{
;
; I:	vorher alle SET-Befehle Drive, Track, Sektor, DMA
; O:	log.Sektor von BDOS DMA nach Buffer
;	erst wenn Buffer ungueltig ist erfolgt der
;	physikalische Laufwerkszugriff
;
write: 	mov  a,c       	; Block schreiben
		sta  dirflg    	; BDOS-FLAG
		call calcmp
		cnz  wload
		call bufcal
		xchg
		call copy		; Record in DMA Buffer
		mvi  a,1h
		sta  wrtflg    	; Buffer wider gueltig
		lda  dirflg
		cpi  1h        	; DIR-Record?
		mvi  a,0h		; ok laden fuer den Ausstieg
		rnz            	; nein: RET mit ACC=00h (=o.k.)
		call putsub		; DIR sichern
		xra  a
		sta  wrtflg    	; Buffer leer o.k.:ACC=00h
		ret            	; ==> zurueck zum BDOS
;
wload: 	lda  wrtflg
		ora  a         	; Buffer leer?
		jz   wlod      	; ja: JMP
		call puttrk    	; sonst abspeichern
		lda  iodrv
		call parcop		; Diskettenparameter vorbereiten
wlod:  	call gettrk
		ret
;
putsub:	call puttrk    	; wg.Error Handling
		ret
;		}
;
;
;***	Unterroutinen zu Read/Write Sector
;{
;     GETTRK/PUTTRK (Bufferverwaltung)
;	Block lesen/schreiben 1KB-Buffer
;	mit IDE-Interface
;
;
; I:	datbuf <== iodma BDOS SET-DMA
;	aktdrv <== iodrv BDOS SET-DISK
;	akttrk <== iotrk BDOS SET-Track
;	aktsec <== iosec BDOS SET-Setor
;
;	wrtflg	 	= 0 Buffer ungueltig)
;	trknr	 =	LBA+1	= laufende Spurnummer
;	secnr	 =	LBA	= laufende Sektornummer
;
; O:	Buffer 	= 1KByte Daten, werden dann Recordweise mit DMA ausgetauscht
;	(A)		= 0 fuer OK
;	(CY) 		= 1 Fehler lesen/schreiben
;
; Test ob Laufwerk D ausgewaehlt wurde
;			dann zu den IDE/CF-Routinen
;
gettrk:	lda	iodrv		;
		cpi	03h		; Laufwerk = D?
		jnz	get_fl	; nein weiter mit Floppy
		jmp	get_ide	; ==> zu IDE-Routinen

;
puttrk:	lda	iodrv	;
		cpi	03h		; Laufwerk = D ?
		jnz	put_fl	; nein weiter mit Floppy
		jmp	put_ide	; ==> zu IDE-Routinen
;
;----------------------------------------------------------
;
; Disk schreiben
;
put_fl:	lda  aktdrv    	;
		call parcop		; Diskettenparameter vorbereiten
		lxi  h,secwr	; Eintrag fuer
		jmp  pgsub		; Unterprogramm schreiben
;
; Disk lesen
;
get_fl:	call calc      	; log.Sec auf phy.Sec umrechnen
		call store	      ; akt. Parameter sichern
		lxi  h,secrd	; Unterprogramm lesen
;
; Start schreiben/lesen
;
pgsub: 	shld pgadr     	; Variable-Unterprogramm
		call codbuf	      ; LW Ansprache vorbereiten
pglp:  	push h
		push d
		push b
		call xlat	      ; Test auf 2.Seite
		call save	      ; nsc-buff speichern ???
;
		db   0cdh      	; CALL
pgadr: 	ds   2h        	; FDC-Routine
;					; schreiben/lesen
;
		call resto		; nsc-buff lesen ???
		pop  b
		pop  d
		pop  h
		jc   bdoser    	; CY on: Error
		call nxtsec		; naechsten Sektor
		jnz  pglp		; weiter
		ret
;
;
secrd: 	push h
		lxi  h,secred
		jmp  secsub
;
secwr: 	push h
		lxi  h,secwrt
;
secsub:	shld secadr		; wr/rd eintragen
		pop  h
		shld hlbuff
		lxi  h,0h
		dad  sp
		shld spbuff
		lxi  sp,stacku
		lhld hlbuff
		db   0cdh      	; CALL
secadr:	ds   2h		; schreiben/lesen
		shld hlbuff
		lhld spbuff
		sphl
		lhld hlbuff
		ret
;
;
secred:	mvi  c,rwdhg	; Anzahl Versuche
scrd:  	push h
		push d
		push b
		call secrdx		; von Disk lesen
		pop  b
		pop  d
		pop  h
		rnc            	; CY off: o.k.
		dcr  c
		jnz  scrd
		stc            	; CY on:  Error
		ret
;
;
secwrt:	mvi  c,rwdhg	; Anzahl Versuche
scwr:  	push h
		push d
		push b
		call secwrx		; auf Disk schreiben
		pop  b
		pop  d
		pop  h
		jnc  verify    	; kein WR-Fehler, aber pruefen!
		dcr  c
		jnz  scwr
		stc            	; CY on: Error
		ret
;
;
verify:	push b		; Test ob WR OK
		push d
		push h
		lxi  h,verbuf
		call secred    	; hier keine Fehlerauswertung
		pop  h
		push h
		call compar		; Byte Vergleich
		pop  h
		pop  d
		pop  b
		jnz  scwr
		ret            	; CY off (=o.k.): RET
;		}
;
;
;}
;
;*****************************************************
;
;      Bufferverwaltung (Hilfsroutinen)
;
;*****************************************************
;{
;
; log.Sektor auf Buffer abbilden
;
bufcal:	lxi  h,buffer	; Bufferadresse holen
		lda  iosec		; log Sektor
		ani  7h		; maskieren
		rrc			; /2
		mov  d,a		;
		mvi  a,0h         ;
		rar               ;
		mov  e,a          ;
		mov  a,d          ;
		ani  7fh          ; Maske bit7=0
		mov  d,a          ;
		dad  d            ; Bufferdresse berechnen
		xchg              ;
		lhld iodma        ;
		xchg              ;
		mvi  c,80h     	; 128 Bytes Laenge
		ret
;
; LW-Ansprache vorbereiten Select-Byte
;
; 	bit 0 	= Dichte		0=DD, 1=SD
; 	bit 1,2	= LW 0-3
; 	bit 2,4	= Steprate 6-30ms
; 	bit 5		= Seite		0=Side 0, 1=Side 1
;
; I:	--
; O: 	(B)	= selbyt
;	(C)	= aktdrv
;	(D)	= akttrk
;	(E)	= aktsec
;
;
codbuf:	xra  a         	; Buffer
		sta  wrtflg    	; ungueltig
		lxi  h,buffer	;
		lda  aktsec	      ;
		mov  e,a	      ;
		lda  aktdrv	      ; <-- LW Byte aufbereiten
		rlc	            ;
		ani  6h	      ;
		mov  b,a	      ; --> Selectbyte
		mvi  d,4h      	; 4 Sektoren x256 Bytes ??
		lda  step	      ;
		ora  b	      ; Steprate hinzufuegen
		mov  b,a	      ;
		lda  side	      ; Seite testen
		ora  a	      ;
		rz	            ;
		lda  trksid	      ; max Track pro Seite
		mov  c,a	      ;
		lda  akttrk	      ;
		cmp  c	      ;
		rc	            ; --> akttrk < 79d
		mov  a,b	      ;
		ori  20h	      ; <-- Side bit in Selectbyte
		mov  b,a	      ; setzen
		ret	            ;
;
; Test fuer 2.Seite
; wenn ja
;
;	I=	(C) max Track
;		(B) selectbyte
;	O=	(D) neue Tracknr.
;		(B) selectbyte
; 	                        ;
xlat: 	lda  side	      ; 0 oder 1
		ora  a	      ; Test ob =0 Z=1
		lda  akttrk	      ;
		mov  d,a	      ;
		rz	            ; --> nur Seite 0
		cmp  c	      ;
		rc	            ; -->
		cma	            ; ab Track 79 mit Seite 1
		add  c	      ; zaehlen die Tracks wieder
		add  c	      ; Rueckwaerts
		mov  d,a	      ; --> Tracknummer absteigend
		ret	            ;
;
; log.(0-1f) - in physikalische (1-16) Sektoren wandeln
;
; und BDOS-Parameter in Register
;
calc:  	lda  iosec		;
		rrc	            ; /2
		ani  0ch	      ;
		inr  a	      ; Korrektur Zaehlweise
		mov  c,a	      ;
		lda  iodrv	      ; LW aus BDOS
		mov  b,a	      ;
		lhld  iotrk	      ; Trk aus BDOS
		xchg	      ;
		ret	            ;
;
calcmp:	call calc
;
; Vergleich ob Parameter noch gueltig
;
comp:  	lda  aktdrv
		cmp  b
		rnz			; -> iodrv ungl. aktdrv
		lhld	akttrk
		mov	a,l
		cmp	e		; akttrk low byte
		rnz			; -> iotrk ungl. akttrk
		mov	a,h		; akttrk high byte
		cmp	d
		rnz			; -> iotrk ungl. akttrk
		lda  aktsec
		cmp  c		; -> iosec ungl. akttsec
		ret
;
; 256d Bytes vergleichen
; fuer Disk write
;
compar:	mvi  c,0h      	;  256d Bytes vergleichen
		push h
		lxi  h,verbuf
		xchg
		pop  h
comlop:	ldax d
		cmp  m
		rnz
		inx  h
		inx  d
		dcr  c
		jnz  comlop
		ret
;
; aktuelle Parameter sichern
;
store: 	mov  a,b
		sta  aktdrv
		xchg
		shld	akttrk
		mov  a,c
		sta  aktsec
		ret
;
; LW Parameter kopieren
;
parcop:	push psw		;
		push h
		add  a
		add  a
		mov  e,a
		mvi  d,0h
		lxi  h,drv0
		dad  d
		lxi  d,step
		mvi  c,3h
		call copy
		pop  h
		pop  psw
		ret
;
;
; allgemeine Kopierroutine
; (HL) Quelle, (DE) Ziel
;
copy:  	mov  a,m
		stax d
		inx  h
		inx  d
		dcr  c
		jnz  copy
		ret
;
nxtsec:	push d
		lxi  d,100h
		dad  d         	; Adresse +256d
		pop  d
		inr  e         	; Sektor-Nr.+1
		dcr  d         	; Sektor-Zaehler -1
		ret
;
save:  	lda  66h
		sta  nscbuf
		ret
;
resto: 	lda  nscbuf
		sta  66h
		ret
;
bdoser:	pop  psw       	; wg. 2xCALL
		pop  psw
		xra  a
		inr  a
		ret            	; Err.:ACC=01h
;
;}
;
;
;*****************************************************
;
;      Floppy-Routinen
;
;*****************************************************
;{
;
comreg   equ	48h
trkreg   equ	49h
secreg   equ	4ah
datreg   equ	4bh
status   equ	comreg
ctrl     equ	40h
;
;
intrd: 	in   datreg
		stax d
		inx  d
		ret
;
;
intwr: 	ldax d
		out  datreg
		inx  d
		ret
;
;
inton: 	push b
		mvi  a,1eh     ; 8085-RST5.5 enable
		mvi  b,02h     ; NSC800-RSTC enable
		jmp  intcte
intof: 	push b
		mvi  a,1fh     ; 8085-RST5.5 disable
		mvi  b,00h     ; NSC800-RSTC disable
intcte:	sim
		nop            ; wg.NSC800-JR-Befehl
		mov  a,b
		out  0bbh
		pop  b
		ret
;
; Sektorzugriff vorbereiten
; Int-Routinen TRAP u. RST.5
;              im RAM erzeugen
;
init:  	di             	;
		push b
		shld datadr
		mvi  a,0e9h    	; PCHL
		sta  24h       	; 8085-TRAP
		sta  66h       	; NSC-TRAP
		lxi  h,2ch     	; RST.5/RSTC
		mvi  m,0f1h    	; POP PSW
		inx  h
		mvi  m,0dbh    	; IN
		inx  h
		mvi  m,status
		inx  h
		mvi  m,0c9h    	; RET
;
		mov  a,d		; trknr aus xlat
		sta  trknr		; in Zwischenspeicher
		mov  a,e		;
		sta  secnr		;
		mov  a,b		;
		sta  selbyt		; Zeiger auf lwnr
		rrc               ;
		ani  3h           ;
		mov  e,a          ;
		mvi  d,0h         ;
		lxi  h,lwnr       ;
		dad  d		; setzen
		mov  c,m          ; holen
		mov  a,b          ;
		ani  21h          ;
		ora  c            ;
		sta  selout       ;
		lxi  h,track0     ;
		dad  d            ;
		pop  b            ;
		ret               ;
;                             ;
lwnr:  	db   02h		; LW 0 (A)
		db   04h		;  .
		db   08h		;  .
		db   10h		; LW 3 (D)
;
;
secwrx:	call init      	; Sektor schreiben
		push h
		lxi  h,intwr
		shld intprg
		pop  h
		lda  trknr
		mov  d,a
		mvi  a,2ah
		cmp  d         	; Sec.kleiner 42d?
		mvi  a,0a2h    	; *./.A6h; WRSEC
		jnc  scwrx
		mov  b,a
		lda  selbyt
		ani  01h
		mov  a,b
		jnz  scwrx
		ani  0fdh
scwrx: 	sta  rwbef
		jmp  start
;
;
secrdx:	call init      	; Sektor lesen
		push h
		lxi  h,intrd
		shld intprg
		pop  h
		mvi  a,80h     	; *./.84h; RDSEC (+30 ms)
		sta  rwbef
;
start: 	mov  a,m
		rlc
		cnc  restor    	; MSB off: Spur-Nr.ungueltig
		mov  a,m
		ani  7fh
		out  trkreg
		lda  trknr
		out  datreg
		mvi  b,10h     	; *./.14h; SEEK
		call posit     	; Spur einstellen
		rc             	; CY on: RET (Fehler)
;
		mvi  c,rwdhg   	; 5 Versuche
wdhg:  	push b
		push h
		call rdwr      	; Schreib/Lese-Operation
		pop  h
		pop  b
		ani  5ch       	; Status auswerten
		jz   ende
		dcr  c
		jnz  wdhg
		jmp  fdcerr
;
ende:  	push psw
		call intof
		pop  psw
		ret
;
;
rdwr:  	lhld datadr
		push h
		pop  d
		lhld intprg
		lda  secnr
		out  secreg
		call inton
		lda  rwbef
		out  comreg
rdwrlp:	ei
		hlt
		jmp  rdwrlp
;
;
restor:	mvi  b,0h      ;	*./.04h; RESTORE
;
posit: 	call pos       ;	Spur einstellen
		ani  18h       ;	Statusfehler?
		jnz  fdcerr    ;	ja: JMP
		in   trkreg
		mov  c,a
		lda  trknr
		cmp  c         ;	gewuenschte = aktuelle Spur?
		jnz  fdcerr    ;	nein: JMP
fdcok: 	ori  80h       ;	MSB on: Spur-Nr.gueltig
		mov  m,a
		xra  a         ;	CY off: kein Fehler
		ret
fdcerr:	mvi  m,0h      ;	MSB off: Spur-Nr.ungueltig
		stc            ;	CY on: Fehler
		ret
;
pos:   	call inton     ;	Kopf posit.;RET:Status im ACC
		call dely1     ;	wg.Hitachi-Timing
		lda  selbyt
		rrc
		rrc
		rrc
		ani  3h        ;	Step Rate
		ora  b         ;	mit CMD verknuepfen
		out  comreg
poslop:	in   status
		rlc            ;	Motor on?
		jnc  poslop    ;	nein: JMP
;
		lda  selout
		out  ctrl
		ei
;
		mvi  c,timout
timlop:	lxi  d,0h      ;	TIMOUT x 0,52 s
timlp: 	dcx  d
		mov  a,d
		ora  e
		jnz  timlp
		dcr  c
		jnz  timlop
;
		mvi  a,0d0h    ;	*(FORCE INTERRUPT)
		out  comreg    ;	hier: Befehl abbrechen
		call intof
		mvi  a,0ffh    ;	Pseudo-Status: Error
		ret
;
;
dely1: 	push psw
		mvi  a,0c0h    ;	192d
delop: 	dcr  a
		jnz  delop
		pop psw
		ret
;}
;
;
;****************************************************
;
;      Kalt- und Warmstart
;
;****************************************************
;{
;
cboot: 	mvi  a,1h      ;Kaltstart
		out  rambnk    ;Bank 1 allein
		lxi  sp,stackc
;
		lxi  d,iobyte
		lxi  h,sysprt
		mov  a,m
		stax d         ;I/O-Byte laden
		inx  d
		inx  h
		mov  a,m
		stax d         ;Kalt-Disk/User laden
;
		mvi  a,0ffh
		sta  aktdrv    ;letztes Drive undefiniert
		mvi  a,0h
		sta  wrtflg    ;Buffer leer
		lxi  h,0h
		shld track0    ;Spur-Regs.undefiniert
		shld track2
;
		lxi  h,msg1    ;*"This is..."
		call msgout
;
gocpm: 	mvi  a,0c3h    ;Initialisierung Page 0
		sta  0h
		sta  5h
		lxi  h,bios
		lxi  d,3h
		dad  d         ;H&L: Warmstart-Adresse
		shld 1h
		lxi  h,bdos
		shld 6h
		lxi  b,buff
		call setdma
		lda  cdisk
		mov  c,a
		jmp  ccp
;
wboot: 	lxi  sp,stackw
		lda  wrtflg
		ora  a         ;ist alter Buffer gerettet?
		jz   load      ;ja: JMP
		call botsub    ;sonst abspeichern
		lda  iodrv
		call parcop
;
load:  	mvi  a,0h
		sta  wrtflg    ;Buffer leer
		mvi  c,nwarm   ;nur (!) CCP und BDOS neu laden
		lxi  d,begin   ;Warmstart von Disk
		lxi  h,ccp
bloop: 	push d
		push h
		push b
		lda  drv0
		mov  b,a       ;Select-Byte
		call secrd
		pop  b
		pop  h
		lxi  d,0100h
		dad  d         ;Adresse +256d
		pop  d
		ora  a
		jc   load      ;Lesefehler: JMP
		inr  e
		dcr  c
		jz   gocpm     ;Start
;
		mov  a,e
		cpi  11h
		jnz  bloop
		inr  d         ;naechste Spur,
		mvi  e,1h      ;erster Sektor
		jmp  bloop
;
;
botsub:	call puttrk    ;wg.Error Handling
		ret
;}
;
;*****************************************************
;
;      Textausgabe
;
;*****************************************************
;{
;
msgout:	mov  a,m
		ora  a
		rz
		mov  c,a
		push h
		call conout
		pop  h
		inx  h
		jmp  msgout
;
msg1:  	db	0dh
		db	0ah
		db	0ah
		db	"This is 60 K CP/M Rel. 2.2"
		db	0dh
		db	0ah
		db	"BIOS V12.62 mit IDE"
		db	0dh
		db	0ah
		db	00h
;}
;
;
;####################################################################
; Modul fuer IDE-Karte
;
string	equ	msgout	; Zuweisung der Tetausgabe
;
;{
; Entwickelt aus IDETOOL von Alfred Klichs 2022,;
; Paul's 8051 Code Library, Paul Stoffregen 2005
; https://www.pjrc.com/tech/8051/ide/
;
;
; Assembler 8085-Simulator-ID V4.93 Oshonsoft (Vladimir Soso)
; Editor 	Notepad++ TAB=6
;
;
; Stand:
;	24.01.2023	aus FDC-0123 vom 23.01.2023
;	07.08.2023	Kapselung verbessert
;
;
; IDE-karte lesen/schreiben
;  Laufwerk 3 [D] = IDE-Karte
;
; Aufruf:
;
;   get_ide		; von IDE lesen (in Buffer ablegen)
;   put_ide		; auf IDE schreiben (von Buffer auf IDE)
;
; Datenuebergabe:
;
;   globale Variable
;
;	buffer <== 1k Buffer fuer den Datenaustausch
;			BDOS <--> BIOS
;
;
;	akttrk <== iotrk BDOS SET-Track
;	aktsec <== iosec 1.Sektor in Moppelformat
;			 wird im AVR-Teil auf 512B/Sektor umgerechnet
;
;	wrtflg <== wird auf 0 gesetzt Buffer ungueltig
;
;   lokale Variable
;
;	idebuf <==	datbuf	= aktuelle Bufferadresse
;	idesec <==	secnr		= laufende Sektornummer
;	idetrk <==	trknr		= laufende Spurnummer
;
;
;	LBA	= Sektor
;	LBA+1	= Track lsb
;	LBA+2	= Track msb
;	LBA+3	= Head
;
; Rueckgabe:
;	Buffer 	= 1KByte Daten, werden dann Recordweise mit DMA ausgetauscht
;	(A)		= 0 fuer OK
;	(CY) 		= 1 Fehler lesen/schreiben
;
;
; Externes Programm
;
;	string	; (HL) Startadresse fuer Tetstring
;			; Textausgabe bis Ende = 00h
;
;
;--------------------------------------------------------------------
;
; IO-Adressen fuer PIO 8255
;
PIO_IDE		EQU	058H		; PIO I/O Adresse
;
PIO_LSB		EQU	0 + PIO_IDE ; Port A
PIO_MSB		EQU	1 + PIO_IDE ; Port B
PIO_CTL		EQU	2 + PIO_IDE ; Port C
PIO_CFG		EQU	3 + PIO_IDE ; I/O modus
;
PIO_RDW		EQU	10010010B 	; Port A+B input, Port C output
PIO_WR		EQU	10000000B 	; Port A+B+C output
;
IDE_DRV		EQU	11100000B 	; select the device
			; x1x00000b LBA master
			; x1x10000b LBA slave
			; x0x00000b CHS master
			; x0x10000b CHS slave
;			; Kanal B als Ausgabe
;
;--------------------------------------------------------------------
;
; Variable fuer IDE Funktionen
;
IDE_A0		EQU	00000001B 	; A0 direct to IDE HDD
IDE_A1		EQU	00000010B 	; A1 dto.
IDE_A2		EQU	00000100B 	; A2 dto.
IDE_CS0		EQU	00001000B 	; CS0 to Inverter
IDE_CS1		EQU	00010000B 	; CS1 dto.
IDE_WR		EQU	00100000B 	; WR dto.
IDE_RD		EQU	01000000B 	; RD dto.
IDE_RST		EQU	10000000B 	; RST dto.
;
IDE_DATA		EQU	IDE_CS0
IDE_ERR		EQU	IDE_CS0 + IDE_A0
IDE_SEC_CNT		EQU	IDE_CS0 + IDE_A1
IDE_SECTOR		EQU	IDE_CS0 + IDE_A1 + IDE_A0
IDE_CYL_LSB		EQU	IDE_CS0 + IDE_A2
IDE_CYL_MSB		EQU	IDE_CS0 + IDE_A2 + IDE_A0
IDE_HEAD		EQU	IDE_CS0 + IDE_A2 + IDE_A1
IDE_COMMAND		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_STATUS		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_CONTROL		EQU	IDE_CS1 + IDE_A2 + IDE_A1
IDE_ASTATUS		EQU	IDE_CS1 + IDE_A2 + IDE_A1 + IDE_A0
;
IDE_CMD_RECAL	EQU	010H
IDE_CMD_READ	EQU	020H
IDE_CMD_WRITE	EQU	030H
IDE_CMD_INIT	EQU	091H
IDE_CMD_ID		EQU	0ECH
IDE_CMD_SPINDN	EQU	0E0H
IDE_CMD_SPINUP	EQU	0E1H
;
;
;===========================================================
;
; initrw =	Buffer, Spur, Sektorregister vorbereiten
;
; CHS_LBA uebertraegt die CHS zu LBA
;
; Parameteruebergabe:
;
;	idebuf	= aktuelle Bufferadresse
;
;	LBA		= Sektor
;	LBA+1		= Track lsb
;	LBA+2		= Track msb
;	LBA+3		= Head
;
initrw:	xra  	a		; clear wrtflg -"Buffer ungueltig"
		sta  	wrtflg	;
		di			; Interrupt sperren
		sta	lba+2
		sta	lba+3		; Head 0
		lxi	h,buffer	;
		shld	idebuf	;
		lhld	akttrk	;
		shld	idetrk		;
		shld	lba+1		; Track lsb/msb
		lda	aktsec	;
		sta	idesec		;
		sta	lba		; Sektor
		lxi	h,buffer	;
		shld	idebuf	;
		ret			;
;
; get/put_cf =	Block von CF lesen/schreiben
;
get_ide:	call	calc		; log.Sektor auf phy. umrechnen
		call	store	      ; akt. Parameter sichern
		call 	initrw	;
		call 	READ_SECTOR	;
		ana	a		;
		jnz	cf_err	; Fehler
		;
		call 	updsek	; 2.Sektor laden
		call	READ_SECTOR	;
		ana	a		;
		jnz	cf_err	; Fehler
		xra	a		;
		ret			; zurueck zum Disk-Manager
		;
		;
put_ide:	call 	initrw	;
		call 	WRITE_SECTOR		;
		ana	a		;
		jnz	cf_err	; Fehler
		;
		call 	updsek	; 2.Sektor laden
		call	WRITE_SECTOR		;
		ana	a		;
		jnz	cf_err	; Fehler
		xra	a		;
		ret			; zurueck zum Disk-Manager
;
;
; updsek/sp = Sektor und Spurnummern aktualisieren
;             und an CF-Karte senden
;
;	! maimal 32 Records pro Spur !
;
updsek:	lxi	h,buffer+512
		shld	idebuf
		lda	idesec	; Sektor holen
		cpi	16		; letzter Sektor spur
		jz	updsp		;
		inr	a		; naechster Sektor gleiche Spur
		sta	idesec	;
		sta	lba
		ret

updsp:	mvi	a,01h		; naechste Spur mit Sektor 1
		sta	idesec		;
		sta	lba
		lhld	idetrk		;
		inx	h		;
		shld	idetrk		;
		shld	lba+1		;
		ret
;
; Fehler (A)=1
;
cf_err:	mvi	a,01		;
		stc			;
		ret			; Zurueck mit Fehler
;
;
;--------------------------------------------------------------------
;
ideinit:	call	IDE_HARD_RESET
		call	IDE_INIT
		jnz	nohd			; IDE Busy
		ret			;

nohd:		lxi	h,nomsg		; keine HD
		call	string		;
		ret

;
IDE_HARD_RESET:
		MVI	A, PIO_RDW		; PIO Read/Write Mode
		OUT	PIO_CFG
		MVI	A, IDE_RST		; hard reset drive
		OUT	PIO_CTL
		PUSH	B
		LXI	B,3000		; ca. 10 ms delay
WLOOP:
		DCX	B
		MOV	A,B
		ORA	C
		JNZ	WLOOP
		POP	B
		XRA	A			; no ide control lines asserted
		OUT	PIO_CTL
		RET
;
; HD Init
;
IDE_INIT:
		call	IDE_GET_ASTATUS	; Test ob Karte
		ana	a			; gesteckt ist
		jz	IDE_NOT_RDY		; nein --> Ende

		MVI	A,IDE_HEAD		;
		MVI	C,IDE_DRV		; select the device
		CALL	IDE_WR_8		;
		LXI	D,0010h		; wait for ready
IDE_RDY_BSY:
		MVI	B,10h
INILP:					; delay for slow drives
		DCR	B
		JNZ	INILP			; djnz $ Z80
		DCX	D
		MOV	A,D
		ORA	E
		JZ	IDE_NOT_RDY		; timeout
		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C

		ANI	01000000B		; bit 6,a
						; Z80, if on then ready
		JZ	 IDE_RDY_BSY	; wait for READY bit to be set
		MOV	A,C

		ANI	10000000B		; bit 7,a
						; Z80, if on then busy
		JNZ	 IDE_RDY_BSY	; wait for BUSY bit to be clear
		RET

IDE_NOT_RDY:
		MVI	a,0FFh
		ORA	A
		STC				; timeout
		RET

;
;
; Wait for DRQ
;
IDE_WAIT_DRQ:
		MVI	A,0FFH
		MOV	B,A
		STA	DELAY_DRQ
MORE_DRQ:

		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C
		ANI	10001000B
		CPI	00001000B
		JZ	DONE_DRQ
		DCR	B
		JNZ	MORE_DRQ
		LDA	DELAY_DRQ
		DCR	A
		JNZ	MORE_DRQ
		STA	DELAY_DRQ
		STC
		RET
DONE_DRQ:
		XRA	A
		RET

DELAY_DRQ:
		DB	0

;
; Wait for BUSY to be clear
;
IDE_WAIT_NOT_BUSY:
		MVI	A,0FFH
		MOV	B,A
		STA	DELAY_BUSY
MORE_BUSY:
		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C
		ANI	11000000B
		XRI	01000000B
		JZ	NOT_BUSY
		DCR	B
		JNZ	MORE_BUSY
		LDA	DELAY_BUSY
		DCR	A
		STA	DELAY_BUSY
		JNZ	MORE_BUSY
		STC
		RET
NOT_BUSY:
		XRA	A
		RET

DELAY_BUSY:
		DB	0

;
; Get Active Status of the IDE Drive
;
IDE_GET_ASTATUS:
		MVI	A,IDE_ASTATUS
		CALL	IDE_RD_8
		MOV	A,C
		RET

;
; 8 bit read
;
IDE_RD_8:
		MOV	D,A			; save adress to D
		OUT	PIO_CTL		; drive address onto control lines
		ORI	IDE_RD		; set read pin on
		OUT	PIO_CTL
;		nop				; delay 2x nop
;		nop
		IN	PIO_LSB		; read lower byte
		MOV	C,A			; to C
		MOV	A,D			; reset read pin first
		OUT	PIO_CTL
		XRA	A			; reset all control pins
		OUT	PIO_CTL
		RET

;
; 8 bit write
;
IDE_WR_8:
		MOV	D,A			; save adress to D
						; Command, head, sector, cylinder etc. in C
		MVI	A, PIO_WR
		OUT	PIO_CFG		; PIO Write Mode
		MOV	A,C
		OUT	PIO_LSB		; Drive lower lines
		MOV	A,D			; Drive address onto control lines
		OUT	PIO_CTL
		ORI	IDE_WR		; set write pin on
		OUT	PIO_CTL
;		nop				; delay	2x nop
;		nop
		MOV	A,D
		OUT	PIO_CTL		; reset write pin first
		XRA	A
		OUT	PIO_CTL		; reset all control pins
		MVI	A,PIO_RDW
		OUT	PIO_CFG		; set PIO read/write mode
		RET

;
; Do the identify command and fill
; the buffer with the drive infos
;
DRIVE_ID:
		CALL	IDE_WAIT_NOT_BUSY
		MVI	A,IDE_HEAD
		MVI	C,IDE_DRV		; select the device
		CALL	IDE_WR_8
		CALL	IDE_WAIT_NOT_BUSY
		MVI	A,IDE_COMMAND
		MVI	C,IDE_CMD_ID	; 0ech
		CALL	IDE_WR_8		; issue the command
		CALL	IDE_WAIT_DRQ
		CALL	READ_DATA
		RET

;
; 16 bit read
;
READ_SECTOR:
						; hdread:
		CALL	IDE_WAIT_NOT_BUSY
		CALL	WR_LBA		; set sector
		MVI	A,IDE_COMMAND	; command register
		MVI	C,IDE_CMD_READ
		CALL	IDE_WR_8
		CALL	IDE_WAIT_DRQ
		CALL	READ_DATA
		CALL	IDE_WAIT_NOT_BUSY
		ANI	10001001B		; busy, DRQ or error
		RZ				; no
		JMP	GET_ERR

;
; 16 bit write
;
WRITE_SECTOR:
						; hdwrite:
		CALL	IDE_WAIT_NOT_BUSY
		CALL	WR_LBA		; set sector
		MVI	A,IDE_COMMAND	; command register
		MVI	C,IDE_CMD_WRITE
		CALL	IDE_WR_8
		CALL	IDE_WAIT_DRQ
		CALL	WRITE_DATA
		CALL	IDE_WAIT_NOT_BUSY
		ANI	10001001B		; busy, DRQ or error
		RZ				; no
						;
						; Return with errors if occurs
GET_ERR:
		MVI	A,IDE_ERR
		CALL	IDE_RD_8
		MOV	A,C
		ORA	A			; set Zero Flag
		RET

;
;
; Read one sector of 512 bytes
;
READ_DATA:	MVI	B,0			; 256 words, 512 bytes per sector
		lhld	idebuf		; target
READ_LP:
		MVI	A,IDE_DATA		; drive address onto control lines
		OUT	PIO_CTL		; and set read pin off (loop)
		ORI	IDE_RD
		OUT	PIO_CTL		; set read pin on
		IN	PIO_LSB		; read the lower byte
		MOV	M,A
		INX	H
		IN	PIO_MSB		; read the upper byte
		MOV	M,A
		INX	H
		DCR	B			; Z80 djnz read_lp
		JNZ	READ_LP
		XRA	A			; set all control pins of
		OUT	PIO_CTL		;
		shld	idebuf		; Zeiger sichern
		ret
;
; Write one sector of 512 bytes
;
WRITE_DATA:	MVI	B,0			; 256 words, 512 bytes per sector
		lhld	idebuf		; target			; source
		MVI	A, PIO_WR
		OUT	PIO_CFG		; PIO Write Mode
WRITE_LP:
		MOV	A,M
		OUT	PIO_LSB		; Drive lower lines with lsb
		INX	H
		MOV	A,M
		OUT	PIO_MSB		; Drive upper lines with msb
		MVI	A,IDE_DATA
		OUT	PIO_CTL		; Drive address onto control lines
		ORI	IDE_WR		; set write pin on
		OUT	PIO_CTL
		MVI	A,IDE_DATA
		OUT	PIO_CTL		; reset write pin
		INX	H
		DCR	B			; Z80 djnz write_lp
		JNZ	WRITE_LP
		XRA	A			; set all control pins of
		OUT	PIO_CTL
		MVI	A,PIO_RDW		; PIO Read/Write Mode
		OUT	PIO_CFG
		shld	idebuf		; Zeiger sichern
		RET
;
;
; LBA = 28bit 4+3x8
; If CHS mode head 8-bit (0-254), sektor 6-bit (1-63)
; cylinder/track 10-bit (0-1023) = 24-bit / 8,38GB max.
;
;
WR_LBA:	LDA	LBA+3			; 4/8-bit, CHS head 0-254
		ANI	0FH			; LBA 0-15
		ORI	IDE_DRV		; mode master/slave CHS/LBA
		MOV	C,A
		MVI	A,IDE_HEAD
		CALL	IDE_WR_8

		LDA	LBA+2			; 8-bit, CHS cyl msb
		MOV	C,A
		MVI	A,IDE_CYL_MSB
		CALL	IDE_WR_8

		LDA	LBA+1			; 8-bit, CHS cyl lsb
		MOV	C,A
		MVI	A,IDE_CYL_LSB
		CALL	IDE_WR_8

		LDA	LBA			; 8-bit, CHS sector

;		and	3fh			; 1-63 if CFC/CHS
		MOV	C,A
		MVI	A,IDE_SECTOR
		CALL	IDE_WR_8

		MVI	C,1			; transfer one sector
		MVI	A,IDE_SEC_CNT
		CALL	IDE_WR_8
		XRA	A
		RET
;
;
;
;--------------------------------------------------------------------
;
;
NOMSG:	DB	"No HDD found!"
		db	0ah		;
		db	0dh		;
		db	00h		;

;					;
lba:		ds	4		; Sektor
;					; Track lsb
;					; Track msb
;					; Head
;                             ;
idebuf	ds	2           ; Zeiger auf Buffer
idesec 	ds	1           ;
idetrk 	ds	2           ;



;####################################################################
;}
;
;*****************************************************
;
;     Variablen-Bereich
;	Die Daten werden vom Bootloader aus der Systemspur
;	Tr,00, Sec.04 hier abgelegt
;
;*****************************************************
;{
;
		org    syspar  ;vom Urloader
;
xlt0:  	ds   20h       ;Translate Tables
xlt1:  	ds   20h
xlt2:  	ds   20h
xlt3:  	ds   20h
;
dpb0:  	ds   10h       ;Disk Parameter Blocks
dpb1:  	ds   10h
dpb2:  	ds   10h
dpb3:  	ds   10h
;
sysprt:	ds   1h        ;I/O-Byte
sysusr:	ds   1h        ;Kaltstart-Drive/User
		ds  0eh
		ds   1h
;
; Drive Parameter
; Bedeutung pro Laufwerk
;
; D0 = Typ	(00=40Tr/1k; 01=80Tr/2k, 02=2x80Tr/2k, 03=2x80Tr/4k
; D1 = Step	(00=6ms, 08=12ms, 10=20ms,18=30ms)
; D2 = Side	(00=ein, 01=zwei Schreib/Lese-Koepfe)
; D3 = Tracks (Spuren pro Seite, max 4Fh=79d)
;
;Dive Parameter
drv0:  	ds   4h
drv1:  	ds   4h
drv2:  	ds   4h
drv3:  	ds   4h
;
sysjmp:	ds   1h        ;Codes umkopieren Y/N
sysvid:	ds   1h
		ds  0eh
;
usadr0:	ds   2h
usadr1:	ds   2h
usadr2:	ds   2h
usadr3:	ds   2h
usadr4:	ds   2h
usadr5:	ds   2h
usadr6:	ds   2h
usadr7:	ds   2h
;
;
		org    	variab
;
trknr: 	ds   1h        ;FDC-Init
secnr: 	ds   1h
selbyt:	ds   1h
selout:	ds   1h
datadr:	ds   2h
track0:	ds   1h        ;Spur-Regs.
track1:	ds   1h
track2:	ds   1h
track3:	ds   1h
rwbef: 	ds   1h        ;RD/WR-Befehl
intprg:	ds   2h        ;RD/WR-Adresse
		ds   3h
;
dirflg:	ds   1h        ;von WRITE
wrtflg:	ds   1h        ;WR erl.:00h
nscbuf:	ds   1h
hlbuff:	ds   2h
spbuff:	ds   2h
		ds   1h
;
iodrv: 	ds   1h        ;SELDSK-Init.(LW-Nr.)
step:  	ds   1h
side:  	ds   1h
trksid:	ds   1h
		ds   4h
;
iotrk: 	ds   2h        ;SETTRK
iosec: 	ds   2h        ;SETSEC
iodma: 	ds   2h        ;SETDMA
		ds   2h
;
aktdrv:	ds   1h        ;STORE-Err.
akttrk:	ds   2h
aktsec:	ds   1h
		ds   5h
;
dirbuf:	ds   80h       ;DIR Access Buffer
;
alv0:  	ds   28h       ;27h fuer 2x80 Tracks
csv0:  	ds   20h       ;20h fuer 2x80 Tracks
alv1:  	ds   28h
csv1:  	ds   20h
alv2:  	ds   28h
csv2:  	ds   20h
alv3:  	ds   28h
csv3:  	ds   20h
;
;}
end
;