;##########################################################
;
;      BOOTCP  CP/M-Boot-Loader  V 11.62
;	 (C) HMS 1986 R.Goessler
;          ;
;      Stand:   Do,18.12.86;10:24:40h                                             ;
;----------------------------------------------------------
;	Quellcode mit Notepad++ (TAB6) an 8085-IDE angepasst
;	03.07.2023 werner.roemer@t-online
;
; 	17.07.2023	Fehlerroutine geaendert
;			Funktion mit FDC Vx.5
;	07.08.2023	IDE LW [D] mit 8MB als Type 3 angelegt
;
;
;
;	Benoetigt werden folgende Systemspuren:
;
;	Tr.00	,Sec.01 -> 2D00...2DFFh	; Batch-Vorspann
;		,Sec.02 -> 2800...29FFh	; dieser Bootloader
;		,Sec.04 -> 2A00...2AFFh	; Systemparameter *
;		,Sec.05 -> 2B00...2CFFh	; Funktionscode
;		,Sec.09 -> 3800...3FFFh	; Videocodec (!89er Hardware)
;
;	Tr.01	,Sec.01 -> D400...DBFFh	; 60k CCP
;		,Sec.09 -> DC00...E9FFh	; 60k BDOS
;	Tr.02	,Sec.07 -> EA00...F3FFh	; 60k BIOS
;
;	Tr.02	,Sec.16 -> 9000...90FFh	; (C)opyright-Vermerk
;			 			; mit F>D01 abrufbar
;	Tr.03	,Sec.01 -> 9000...9FFFh	; Text fuer Laufwerksanpassung
;						; mit F>D02 abrufbar
;
;	* ab 2AD0h stehen die Format-Laufwerkszuordnung
;
;---------------------------------------------------------
;
	org    2800h		; Systemram

;
;
ci    	equ	43h		; Console Input
co    	equ	49h         ; Console Output
loinit	equ	58h         ; Drucker auf V24
copy  	equ	88h         ; Speicher kopieren
;     	equ               ;
mowarm	equ	1003h       ; Videomonitor Warnstart
string	equ	1021h       ; Zeichenausgabee bis 00h
error		equ	1027h       ; Fehleranzeige (A)
crson 	equ	1036h       ; Curser ein
crsoff	equ	1039h       ; Cursor aus
;                             ;
fdcwrm	equ	2003h       ; nur in FDC Vx.6
rdblok	equ	2009h 	; FDC Vx.6
rdsec 	equ	200fh 	; FDC Vx.5
;
rambnk	equ	24h   	; Latch A16...A18
vidbnk	equ	2ah   	; RAM-Zeichen-Generator
;
;
drvpar	equ	29c0h 	; Drive-Parameter
drvofs	equ	39c0h
syspar	equ	2a00h 	; System-Parameter
sysofs	equ	3a00h
fctbuf	equ	2b00h 	; FCT-Codes
;
rambeg	equ	2e9ch 	; FDC-RAM-Zellen
trknr 	equ	2ea8h
;
srcbeg	equ	2fbfh       ; Quelle beginn
srcend	equ	2fc1h       ; Quelle ende
dstbeg	equ	2fc3h       ; Ziel start
;
vidram	equ	3800h 	; Zeichen-Generator
vidbuf	equ	8000h
;
ccp   	equ	0d400h	; 60-K-Version
bios  	equ	0ea00h
sysbuf	equ	0f400h
syslen	equ	  0ffh	; 256 Bytes
;
;
begin 	equ	0101h 	; Trk.1, Sec.1
nsecs 	equ	  1fh 	; 31d Sektoren (ohne (C))
;
;
bootcp:	call	crsoff
		lxi  	h,text0	; "MOPPEL-CP/M-Urload..."
		call	string
		lxi  	h,text1	; Printer...
		call	string
		call	loinit
;
boot0: 	lda  	sysjmp
		ora  	a
		jz  	boot1
		lxi  	h,text2	; Function-Codes...
		call	string
		lxi  	h,fctbuf
		lxi  	d,0005h
		mvi  	c,2h
		call	readsk
;
boot1: 	lda  	sysvid
		ora  	a
		jz  	boot2
		lxi  	h,text3	; Video-Codes...
		call	string
		lxi  	h,vidbuf	; Video-Codes einlesen
		lxi  	d,0009h
		mvi  	c,8h
		call	readsk
		lxi  	b,7ffh
		lxi  	d,vidram
		lxi  	h,vidbuf
botlp: 	mov 	a,m
		stax 	d
		inx  	h
		inx  	d
		dcx  	b
		xra  	a
		cmp 	c
		jnz 	botlp
		cmp 	b
		jnz 	botlp
		mvi  	a,1h
		out 	vidbnk   	;RAM-Chr-Generator aktivieren
;
boot2: 	lxi  	h,text4  	;Bank 1...
		call	string
		mvi  	a,81h
		out 	rambnk   	;Bank 1 parallel schalten
;
		lxi  	h,disk0  	;LW-Daten ermitteln
		lxi  	d,dpb0
		call	partst
		lxi  	h,disk1
		lxi  	d,dpb1
		call	partst
		lxi  	h,disk2
		lxi  	d,dpb2
		call	partst
		lxi  	h,disk3
		lxi  	d,dpb3
		call	partst
;
		lxi  	h,syspar
		shld	srcbeg
		lxi  	d,syslen
		dad  	d
		shld	srcend
		lxi  	h,sysbuf
		shld	dstbeg
		call	copy     	; LW-Daten umkopieren
;
boot3: 	mvi  	c,nsecs  	; CCP/BDOS/BIOS laden
		lxi  	d,begin  	; 31d Sektoren ab Trk.1, Sec.1
		lxi  	h,ccp    	; ins RAM ab CBASE
		call	readsk
		call	crson
		jmp 	bios
;
;
;##########################################################
;
;      Unterstuetzungs-Routinen
;
;##########################################################
;
readsk:	lda 	rdsec		; Test Monitorversion
		cpi 	0c3h
		jz  	readv5	; zu FDC-Routinen Vx.5
		;
		; fuer FDC-Routine [RDBLOK] aus Vx.6
		;
readv6:	mov 	a,c      	; C: Records
		dcr 	a
		add 	h        	; A: RAMBEG+Recs-1=RAMEND
		mov 	l,h      	; L: RAMBEG
		mov 	h,a      	; H: RAMEND
		shld	rambeg
		mov 	l,d      	; L: TRKNR
		mov 	h,e      	; H: SECNR
		shld	trknr
		jmp 	rdblok
;
; Boot ueber [RDSEC] aus Vx.5
;
readv5:	push	d        	; D&E: Trk/Sec
		push	h        	; H&L: RAM-Beginn
		push	b        	; C:   Anzahl Sektoren
		mvi 	b,18h    	; B:   SELBYT
		call	rdsec
		pop 	b
		pop 	h
		lxi 	d,100h
		dad 	d
		pop 	d
		ora 	a
		jnz 	boterr	; --> Fehler
		dcr 	c
		rz			; --> fertig
		inr 	e
		mov 	a,e
		cpi 	11h
		jnz 	readv5
		inr 	d        	; naechste Spur
		mvi 	e,1h     	; erster Sektor
		jmp 	readv5
;
;
boterr:	pop	psw
		lxi	h,text5
		call	string
		jmp	1003h		; zurueck zum Monitor
;
;
; DPBs umkopieren
; unter Berueksichtigung der
; Format --> LW Zuordnung
;
partst:	mvi  	a,3h   	; DPB Format 3
		lxi  	b,typ3	;
		cmp  	m		;
		jz   	parcpy
;
		dcr  	a
		lxi  	b,typ2
		cmp  	m
		jz   	parcpy
;
		dcr  	a
		lxi  	b,typ1
		cmp  	m
		jz   	parcpy
;
		lxi  	b,typ0 	; default drive type
parcpy:	push 	b
		pop  	h      	; H&L: Source
		mvi  	c,10h  	; 16d Bytes
parlop:	mov  	a,m
		stax 	d      	; D&E: Destination
		inx  	h
		inx  	d
		dcr  	c
		jnz  	parlop
		ret
;
;
;##########################################################
;
;      Klartext-Meldungen
;
;##########################################################
;
text0:	db	1bh
		db	16h 		; Cursor pos
		db	05h
		db	27h 		; Ende Zeile 5
		db	0dh 		; Anfang naechste Zeile
		db	1bh
		db	17h 		; Clear Screen
		db	0dh
		db	0dh
		db	"MOPPEL-CP/M-Boot-Loader V 11.62"
		db	0dh
		db	0h
;
text1:	db	0dh
		db	"Printer"
		db	0dh
		db	0h
;
text2:	db	0dh
		db	"Function Codes"
		db	0dh
		db	0h
;
text3:	db	0dh
		db	"Video Codes"
		db	0dh
		db	0h
;
text4:	db	0dh
		db	"Bank 1"
		db	0dh
		db	0h

text5:	db	0dh
		db	"Boot Error"
		db	0dh
		db	0h

;
;
;##########################################################
;
; Format-Typen
;
;	Hiermit werden die möglichen Diskettenformate
;	festgelegt, die Zuordnung zu den vier Laufwerken
;	erfolgt ueber die Laufwerksliste:
;
;	 Drive Parameter
;
;	 D0 = Typ	(00=40Tr/1k; 01=80Tr/2k, 02=2x80Tr/2k, 03=2x80Tr/4k
;	 D1 = Step	(00=6ms, 08=12ms, 10=20ms,18=30ms)
;	 D2 = Side	(00=ein, 01=zwei Schreib/Lese-Koepfe) muss zu D0 passen
;	 D3 = Tracks	(Spuren pro Seite, max 4Fh=79d)
;
;	 disk0: 	ds   4h	; [A]
;	 disk1: 	ds   4h	; [B]
;	 disk2: 	ds   4h	; [C]
;	 disk3: 	ds   4h	; [D]
;
;	siehe unten...
;
;	mit der Routine "partst" (s.o.) wird der jeweilige DPB
;	ins BIOS ab Adresse "SYSBUF" (F400h) kopiert
;
;##########################################################
;
		org    drvpar

;
typ0:		ds   0h   		; 1x40 Tracks (1-K-Bloecke)
;
		dw   0020h		; SPT:Sectors per Track (4K/128)
		db     03h		; BSH:Block Shift (log2<1K/128>)
		db     07h		; BLM:Block Mask       (2^BSH-1)
		db     00h		; EXM:Extension Mask
		dw   008fh		; DSM:Disk Size Mem. (36Tr;4K-1)
		dw   003fh		; DRM:Directory Mem.   (2K/32-1)
		db    0c0h		; AL0:Allocation Vector 0 (2Bl.)
		db     00h		; AL1:Allocation Vector 1  (./.)
		dw   0010h		; CKS:Check Size Vector(DRM+1)/4
		dw   0004h		; OFF:Offset    (4 Systemspuren)
		db     00h		; redundant (Fuell-Byte)
;
typ1: 	dw   0020h		; 2x40/1x80 Tracks (2-K-Bloecke)
		db     04h
		db     0fh
		db     01h
		dw   0097h
		dw   007fh
		db    0c0h
		db     00h
		dw   0020h
		dw   0004h
		db     00h
;
typ2: 	dw   0020h		; 2x80 Tracks (2-K-Bloecke)
		db     04h
		db     0fh
		db     00h
		dw   0137h
		dw   007fh
		db    0c0h
		db     00h
		dw   0020h
		dw   0004h
		db     00h
;
; IDE-Karte 8MB
;
; Parameter Disk3 in 2ADCh als Type 03
;
typ3:		dw	32  		; Sectors per Track
		db	5  		; Block shift
		db	31  		; Block Mask	BLS = 4096
		db	1   		; Extnt Mask
		dw	2047  	; DSM Disk Size-1 = 8MB
		dw	511  		; Directory max.
		db	11110000b 	; Alloc0
		db	0	  	; Alloc1
		dw	0	  	; Check Size
		dw	1	   	; Offset
		db     00h
;
;
;##########################################################
;
;      Systemparameter
;
;##########################################################
;
		org    syspar

;
xlt0:  	ds   20h    ; Translate Tables
xlt1:  	ds   20h
xlt2:  	ds   20h
xlt3:  	ds   20h
;
dpb0:  	ds   10h    ; Disk Parameter Blocks
dpb1:  	ds   10h
dpb2:  	ds   10h
dpb3:  	ds   10h
;
sysprt:	ds   1h     ; CP/M-I/O-Byte
sysusr:	ds   1h     ; CP/M-Kaltstart-Drive/User
sysres:	ds   0eh
;
; Drive Parameter
;
; D0 = Typ	(00=40Tr/1k; 01=80Tr/2k, 02=2x80Tr/2k, 03=2x80Tr/4k
; D1 = Step	(00=6ms, 08=12ms, 10=20ms,18=30ms)
; D2 = Side	(00=ein, 01=zwei Schreib/Lese-Koepfe)
; D3 = Tracks (Spuren pro Seite, max 4Fh=79d)
;
disk0: 	ds   4h	; [A]
disk1: 	ds   4h	; [B]
disk2: 	ds   4h	; [C]
disk3: 	ds   4h	; [D]
;
sysjmp:	ds   1h     ; FCT-Codes umkopieren
sysvid:	ds   1h     ; Video-Codes umkopieren
;
end
;
;##########################################################